// Copyright (C) 2024 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only


#ifndef QTAUDIO_H
#define QTAUDIO_H

#if 0
#pragma qt_class(QtAudio)
#endif

#include <QtMultimedia/qaudio.h>

QT_BEGIN_NAMESPACE

QT_END_NAMESPACE

#endif // QTAUDIO_H
