// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-3.0-only

#ifndef QTSPATIALAUDIOGLOBAL_H
#define QTSPATIALAUDIOGLOBAL_H

#include <QtMultimedia/qtmultimediaglobal.h>
#include <QtSpatialAudio/qtspatialaudioexports.h>

#endif // QTMULTIMEDIAGLOBAL_H
