/* This file was generated by syncqt. */
#ifndef QT_QTMULTIMEDIA_VERSION_H
#define QT_QTMULTIMEDIA_VERSION_H

#define QTMULTIMEDIA_VERSION_STR "6.7.2"

#define QTMULTIMEDIA_VERSION 0x060702

#endif // QT_QTMULTIMEDIA_VERSION_H
